'use strict'

// Imports
const { Shell, Clutter, Meta } = imports.gi;
const Main = imports.ui.main;

// Effect configuration
const EffectBrightness = 1
const EffectOpacity = 150
const EffectSigma = 15

// Blurred window
var BlurredWindow = class BlurredWindow {

  constructor(meta_window, extension) {

    // Initialize
    this.meta_window = meta_window
    this.window_actor = meta_window.get_compositor_private()
    this.extension = extension

    // Register events
    this.idDestroy = this.window_actor.connect('destroy', () => this.onDestroy())
    this.idResize = this.meta_window.connect('size-changed', () => this.onResize())
    this.idVisible = this.window_actor.connect('notify::visible', () => this.onVisible())

    // Create blur effect
    this.blur_effect = new Shell.BlurEffect({
      sigma: EffectSigma,
      brightness: EffectBrightness,
      mode: Shell.BlurMode.BACKGROUND
    })

    // Create blur actor
    this.blur_actor = this.create_blur_actor(this.meta_window, this.blur_effect)

    // Add blur actor and set opacity
    this.window_actor.insert_child_at_index(this.blur_actor, 0)
    this.set_window_opacity(this.window_actor, EffectOpacity);

    // Hide the blur actor if window is invisible
    if (!this.window_actor.visible) {
      this.blur_actor.hide();
    }

  }

  onDestroy() {

    // Notify extension
    this.extension.onWindowDestroyed(this)

  }

  onVisible() {

    // Hide/show blur if window visiblity changed
    if (this.window_actor.visible) {
      this.blur_actor.show();
    } else {
      this.blur_actor.hide();
    }

  }

  onResize() {

    // Resize blur actor
    let allocation = this.compute_allocation(this.meta_window)
    this.blur_actor.x = allocation.x
    this.blur_actor.y = allocation.y
    this.blur_actor.width = allocation.width
    this.blur_actor.height = allocation.height

  }
  
  destroy() {

    // Remove blur and reset opacity
    this.set_window_opacity(this.window_actor, 255)
    this.window_actor.remove_child(this.blur_actor)

    // Unregister events
    this.meta_window.disconnect(this.idRename)
    this.meta_window.disconnect(this.idResize)
    this.window_actor.disconnect(this.idVisible)
    this.window_actor.disconnect(this.idDestroy)

  }

  // Compute the size and position for a blur actor.
  compute_allocation(meta_window) {
    const is_wayland = Meta.is_wayland_compositor()
    const monitor_index = meta_window.get_monitor()
    const scale = is_wayland ? Main.layoutManager.monitors[monitor_index].geometry_scale : 1
    let frame = meta_window.get_frame_rect()
    let buffer = meta_window.get_buffer_rect()
    return {
      x: (frame.x - buffer.x) / scale,
      y: (frame.y - buffer.y) / scale,
      width: frame.width / scale,
      height: frame.height / scale
    }
  }

  // Create a new already blurred widget
  create_blur_actor(meta_window, blur_effect) {
    let allocation = this.compute_allocation(meta_window)
    let blur_actor = new Clutter.Actor({
        x: allocation.x,
        y: allocation.y,
        width: allocation.width,
        height: allocation.height
    })
    blur_actor.add_effect_with_name('blur-effect', blur_effect)
    return blur_actor
  }

  // Set the opacity of the window actor that sits on top of the blur effect
  set_window_opacity(window_actor, opacity) {
    window_actor.get_children().forEach(child => {
        if (child.name !== "blur-actor") {
          child.opacity = opacity;
        }
    })
  }

}
